package com.nisum.example.mongodb.controller;

import com.nisum.example.mongodb.model.Employee;
import com.nisum.example.mongodb.service.IEmployeeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

/**
 * Controller class having http POST, GET, UPDATE and DELETE methods
 */
@RestController
public class EmployeeController {

    @Autowired
    private IEmployeeService employeeService;

    /**
     * POST method to perform record creation
     * @param e
     * @return Mono
     */
    @PostMapping(path = "/create")
   @ResponseStatus(HttpStatus.CREATED)
    public Mono<Employee> create(@RequestBody Employee e) {
        return employeeService.create(e);
    }

    /**
     * GET method to fetch record based on Id
     * @param id
     * @return Mono
     */
    @GetMapping("/{id}")
    public ResponseEntity<Mono<Employee>> findById(@PathVariable("id") Integer id) {
        Mono<Employee> e = employeeService.findById(id);
        HttpStatus status = e != null ? HttpStatus.OK : HttpStatus.NOT_FOUND;
        return new ResponseEntity<Mono<Employee>>(e, status);
    }

    /**
     * GET method to fetch record based on name
     * @param name
     * @return Flux
     */
    @GetMapping("/name/{name}")
    public Flux<Employee> findByName(@PathVariable("name") String name) {

        return employeeService.findByName(name);
    }

    /**
     * GET method to fetch all records
     * @return Flux
     */
    @GetMapping(value = "", produces = MediaType.TEXT_EVENT_STREAM_VALUE)
    public Flux<Employee> findAll() {
        Flux<Employee> emps = employeeService.findAll();
        return emps;
    }

    /**
     * PUT method to do update of record
     * @param e
     * @return Mono
     */
    @PutMapping("/update")
    @ResponseStatus(HttpStatus.OK)
    public Mono<Employee> update(@RequestBody Employee e) {
        return employeeService.update(e);
    }

    /**
     * DELETE method to remove record based on Id
     * @param id
     * @return Void
     */
    @DeleteMapping("/delete/{id}")
    @ResponseStatus(HttpStatus.OK)
    public void delete(@PathVariable("id") Integer id) {
        employeeService.delete(id).subscribe();
    }

}
