package com.nisum.example.mongodb.service;

import com.nisum.example.mongodb.model.Employee;
import com.nisum.example.mongodb.repository.EmployeeRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.time.Duration;

/**
 *Service class having methods for CURD operations
 * Methods having Mono<T> and Flux<T> as return types
 */
@Service
public class EmployeeServiceImpl implements IEmployeeService {

    @Autowired
    private EmployeeRepository employeeRepository;

    /**
     * perform create operation
     * @param e
     * @return Mono
     */
    @Override
    public Mono<Employee> create(Employee e) {
        return employeeRepository.save(e);
    }

    /**
     * perform fetch operation based on ID
     * @param id
     * @return Mono
     */
    @Override
    public Mono<Employee> findById(Integer id) {
        return employeeRepository.findById(id);
    }

    /**
     * perform fetch operation based on name
     * @param name
     * @return Flux
     */
    @Override
    public Flux<Employee> findByName(String name) {
        return employeeRepository.findByName(name);
    }

    /**
     * perform fetch all records
     * @return Flux
     */
    @Override
    public Flux<Employee> findAll() {
        return employeeRepository.findAll().delayElements(Duration.ofSeconds(1));
    }

    /**
     * perform update of object
     * @param e
     * @return updated object of type Mono
     */
    @Override
    public Mono<Employee> update(Employee e) {
        return employeeRepository.save(e);
    }

    /**
     * perform delete based on id
     * @param id
     * @return Mono
     */
    @Override
    public Mono<Void> delete(Integer id) {
        return employeeRepository.deleteById(id);
    }
}
